@@ -13,6 +13,7 @@ if TYPE_CHECKING:
     from collections.abc import Iterable
 
     from polars import Series
+    from polars._plr import PySeries
     from polars._typing import (
         Ambiguous,
         EpochTimeUnit,
@@ -23,7 +24,6 @@ if TYPE_CHECKING:
         TemporalLiteral,
         TimeUnit,
     )
-    from polars.polars import PySeries
 
     if sys.version_info >= (3, 13):
         from warnings import deprecated
@@ -679,6 +679,41 @@ class DateTimeNameSpace:
         ]
         """
 
+    def days_in_month(self) -> Series:
+        """
+        Extract the number of days in the month from the underlying date representation.
+
+        Applies to Date and Datetime columns.
+
+        Returns the number of days in the month.
+        The return value ranges from 28 to 31.
+
+        Returns
+        -------
+        Series
+            Series of data type :class:`Int8`.
+
+        See Also
+        --------
+        month
+        is_leap_year
+
+        Examples
+        --------
+        >>> from datetime import date
+        >>> s = pl.Series(
+        ...     "date", [date(2001, 1, 1), date(2001, 2, 1), date(2000, 2, 1)]
+        ... )
+        >>> s.dt.days_in_month()
+        shape: (3,)
+        Series: 'date' [i8]
+        [
+                31
+                28
+                29
+        ]
+        """
+
     def week(self) -> Series:
         """
         Extract the week from the underlying date representation.
@@ -1828,10 +1863,6 @@ class DateTimeNameSpace:
         - 1q    (1 calendar quarter)
         - 1y    (1 calendar year)
 
-        These strings can be combined:
-
-        - 3d12h4m25s # 3 days, 12 hours, 4 minutes, and 25 seconds
-
         By "calendar day", we mean the corresponding time on the next day (which may
         not be 24 hours, due to daylight savings). Similarly for "calendar week",
         "calendar month", "calendar quarter", and "calendar year".
@@ -1950,10 +1981,6 @@ class DateTimeNameSpace:
         - 1q    (1 calendar quarter)
         - 1y    (1 calendar year)
 
-        These strings can be combined:
-
-        - 3d12h4m25s # 3 days, 12 hours, 4 minutes, and 25 seconds
-
         By "calendar day", we mean the corresponding time on the next day (which may
         not be 24 hours, due to daylight savings). Similarly for "calendar week",
         "calendar month", "calendar quarter", and "calendar year".
